# -*- coding: utf-8 -*-
"""
__author__ = Isfa Rasool
__email__ = isfa.rasool@nmbu.no

This program calculates bending stress in a wind turbine tower.
"""

from pylab import *
import numpy as np
#from math import *
from mik_wafofuncs import fatiguedamage_twoslope
#import matplotlib.pyplot as plt


# Tårnrot (stål)    x1 0 0 13 dcyl1 9.7 tcyl1 0.090 pre_strain_long   0. 
# Midten            x2 0 0 47.8 dcyl2 7.99197530864198 tcyl2 0.0665 btype beam 


# Diameter 
D_trot = 9.7     #  ytre diameter ved tårnrot hentet fra input-fil
D_tmid = 7.9919753 # ytre diameter ved midten av tårnet hentet fra input-fil


d_trot = 9.7       - 2.*0.0900     # indre diameter ved tårnrot
d_tmid = 7.9919753 - 2.*0.0665     # indre diameter ved midten av tårnet


# Treghetsmoment
treg_moment_trot = (pi/64.) * (D_trot**4 - d_trot**4) # annet arealmoment ved tårnrot
treg_moment_tmid = (pi/64.) * (D_tmid**4 - d_tmid**4) # annet arealmoment midt på tårnet


# Innhenting av data for momenter.
#a = np.genfromtxt("tower_root_moment.txt",usecols=(1,2,3,4), skiprows=(13), dtype=None) # # load file momenter ved tårn rot
a = np.genfromtxt("tower_mid_moment_case2.txt", usecols=(1,2,3,4), skiprows=(13), dtype=None) # load file momenter midten av tårnet


# Bøye Spenning
tid = a[:, 0];
Mx = a[:, 3]; # momenter i x-retning, elementets z-akse, global x akse
My = a[:, 2]; # momenter i y-retning, elementets y-akse, global y akse


#Tdegrees = [0, 30, 60, 90, 120, 150, 180, 210, 240, 270, 300, 330]
Tdegrees = np.linspace(0, 330, num=12) # Vinkler der spenning skal beregnes

ntime = size(tid)
ndeg  = size(Tdegrees)
   
bend_stress = zeros([ntime,ndeg],dtype='float')
SCF = 1.61 # for bending stress. From DNV-RP-C203, curve F3


for   itime in range(ntime):
  for ideg  in range(ndeg):
    T = Tdegrees[ideg]
    x = .5*D_tmid*cos(pi*T/180.)
    y = .5*D_tmid*sin(pi*T/180.)
    bend_stress[itime,ideg] = SCF*(-((Mx[itime]/treg_moment_tmid)*y) - ((My[itime]/treg_moment_tmid)*x))
    #print tid[itime],T,x,y,bend_stress[itime,ideg]


# Table 2.2 in DNV RP-C203
# Used curve F3 !

m1    = 3.
loga1 = 11.146
m2    = 5.
loga2 = 14.576
Nlim  = 1.e6

ideg = 0
for ideg  in range(ndeg):
    dt = fatiguedamage_twoslope(tid,bend_stress[:,ideg],m1,loga1,m2,loga2,Nlim,th=25E-3,tref=25E-3,k=0.25)
    #print 'damage during one hour at ',Tdegrees[ideg],' :',dt
    dt20 = dt*177.8*20
    print dt20


figure()
hold(True)
plot(tid,Mx/1.e6,'k-',label='Mx')
plot(tid,My/1.e6,'r-',label='My')
xlabel('t [s]')
ylabel('Moment [MNm]')
title('Tower Mid Bending Moment')
legend(loc='best')
hold(False)
savefig('tower_mid_moment.png',dpi=70)


for ideg  in range(ndeg):
  figure()
  hold(True)
  plot(tid, bend_stress[:,ideg]/1.e6,'k-',label='')
  xlabel('t [s]')
  ylabel('Bend_stress [MPa]')
  title('Bending stress at azimuth '+str(Tdegrees[ideg]))
  legend(loc='best')
  hold(False)
  savefig('mid_tower_stress_'+str(int(Tdegrees[ideg]))+'.png',dpi=70)


show()
